<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +--------------------------------------------------------------------+
// | The Lazym Project                                                  |
// +--------------------------------------------------------------------+
// | Copyright (c) 2004 ູѰ                                      |
// +--------------------------------------------------------------------+
// |  ҽ  Ͽ  LICENSE  LazyBoard ̼ |
// | 1.0   ֽϴ.  ̼    ּҿ  |
// | Ȯΰմϴ.                                                    |
// | http://www.lazybbs.net/license/korean__1_0.txt                     |
// | α׷̳ ̼   gonom9@gmail.com     |
// | Ͻðų http://www.lazybbs.net  湮Ͻñ ٶϴ.            |
// +--------------------------------------------------------------------+
//

/**
 * INI file handling class
 */
class INIFile
{
	//{{{ public
	/**
	 * file header
	 */
	var $header = '';

	/**
	 * file footer
	 */
	var $footer = '';
	//}}}
	//{{{ private
	/**
	 * data
	 * @var array
	 */
	var $_data = array();

	/**
	 * ini file name
	 * @var string
	 */
	//}}}
	var $_filename = null;

	function inifile()
	{
		$this->__construct();
	}
	
	function __construct()
	{
		$this->_data = array();
		$this->_filename = null;
	}

	function &Singleton()
	{
		static $instance;

		if (!isset($instance)) $instance = new INIFile;
		return $instance;
	}

	function Load($filename)
	{
		if (!is_readable($filename)) {
			Lazym::raiseError('"%s" file is not exists!', $filename);
			return false;
		}
		
		$this->_data = array();
		$this->_filename = $filename;
		$data = parse_ini_file($filename, true);
		foreach ($data as $key=>$value) {
			if (!is_array($value)) $value = array();
			foreach ($value as $k=>$v) {
				$this->_data[$key][$k] = $this->_dequote($v);
			}
		}

		return true;
	}

	function Save($filename=null)
	{
		if (is_null($filename)) $filename = $this->_filename;
		if (!is_writable(dirname($filename))) {
			Lazym::raiseError('"%s" directory is not writable!', $filename);
			return false;
		}

		$fp = fopen($filename, 'w');
		if ($this->header) fwrite($fp, $this->header."\n");
		foreach ($this->_data as $key=>$value) {
			fwrite($fp, "[".$key."]\n");
			if (!is_array($value)) $value = array();
			foreach($value as $k=>$v) {
				fwrite($fp, $k.'='.$this->_quote($v)."\n");
			}
		}
		if ($this->footer) fwrite($fp, $this->footer."\n");
		fclose($fp);
		return true;
	}

	function getArray()
	{
		return $this->_data;
	}

	function getData($section, $key, $default=null)
	{
		if (isset($this->_data[$section][$key])) {
			return $this->_data[$section][$key];
		} else {
			return $default;
		}
	}

	function setData($section, $key, $value=null)
	{
		if (isset($this->_data[$section][$key]) && is_null($value)) {
			unset($this->_data[$section][$key]);
		} else {
			$this->_data[$section][$key] = $value;
		}
	}

	function _quote($data)
	{
		$conv = array('#'=>'##', "\n"=>'#n', "\t"=>'#t', '"'=>"#'#'");
		
		if (is_numeric($data)) {
			return $data;
		} else if (is_string($data)) {
			return '"'.strtr($data, $conv).'"';
		} else if (is_array($data)) {
			return '"#array::'.strtr(serialize($data), $conv).'"';
		}
	}

	function _dequote($str)
	{
		$conv = array('##'=>'#', '#n'=>"\n", '#t'=>"\t", "#'#'"=>'"');
		
		if (preg_match('/^#array::(.+)$/', $str, $match)) {
			return unserialize(strtr($match[1], $conv));
		} else {
			return strtr($str, $conv);
		}
	}

	function Free()
	{
		$this->_data = array();
		$this->_filename = null;
	}
}
?>